#imports
import numpy as np
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
from sklearn.metrics import silhouette_score
from sklearn.preprocessing import StandardScaler

#This function performs the k means clustering.
#The variable readfile is the file that will be analyzed by the function.
#The variable convex_hull is a boolean that is True if a convex_hull file was inputed as 
# a readfile.
def kmeans(readfile, convex_hull):
    with open(readfile,'r') as f:
        data=[]
        volumes=[]
        for line in f:
            info = eval(line)
            values=[info[2],info[3]]
            volumes.append(info[2])
            data.append(values)

    # Separate the x and y coordinates into separate lists
    x = [point[0] for point in data]
    y = [point[1] for point in data]

    # Create a scatter plot
    plt.scatter(x, y)

    # Add labels and title
    if convex_hull:
        plt.xlabel('Volume')
        plt.ylabel('Density')
    else:
        plt.xlabel('Average distance')
        plt.ylabel('Total distance')
    plt.title('Scatter Plot')

    # Display the plot
    plt.show()

     # Create an instance of StandardScaler
    scaler = StandardScaler()

    # Fit and transform the data

    # Convert the points to a NumPy array
    if convex_hull!=True:
        normalized_data = scaler.fit_transform(data)
        X = np.array(normalized_data)
    else:
        X = np.array(data)
    X2 = np.array(data)

    # Define the range of cluster numbers to evaluate
    min_clusters = 2
    max_clusters = 9

    # Perform silhouette analysis for different cluster numbers
    best_score = -1
    optimal_clusters = -1

    #For the silhouette analysis
    for k in range(min_clusters, max_clusters+1):
        # Perform k-means clustering
        kmeans = KMeans(n_clusters=k)
        kmeans.fit(X)
        labels = kmeans.labels_

        # Compute the silhouette score
        score = silhouette_score(X, labels)

        # Update the best score and optimal number of clusters
        if score > best_score:
            best_score = score
            optimal_clusters = k

    # Define the number of clusters
    k = optimal_clusters

    # Perform k-means clustering
    kmeans = KMeans(n_clusters=k)
    kmeans.fit(X)

    # Get the cluster labels and cluster centers
    labels = kmeans.labels_
    centers = kmeans.cluster_centers_

    # Count the number of points in each cluster
    counts = np.bincount(labels)

    # Plot the points with different colors for each cluster
    plt.scatter(X2[:, 0], X2[:, 1], c=labels)

    # Plot the cluster centers
    if convex_hull!=True:
        centers=scaler.inverse_transform(centers) #Scale back the centers to fit the original data (non normalized data)
    plt.scatter(centers[:, 0], centers[:, 1], c='red', marker='x')

    # Add labels and title
    if convex_hull:
        plt.xlabel('Volume')
        plt.ylabel('Density')
    else:
        plt.xlabel('Average distance')
        plt.ylabel('Total distance')
    plt.title('Scatter Plot')

    # Display the number of points in each cluster
    for i, count in enumerate(counts):
        plt.text(centers[i, 0], centers[i, 1], f'Count: {count}', color='black', ha='center')

    # Display the plot
    plt.show()

#method1
#kmeans('finalThree/method1/convex_hull_all.txt',True)
#kmeans('finalThree/method1/convex_hull_c_4.txt',True)
#kmeans('finalThree/method1/convex_hull_c_5.txt',True)
#kmeans('finalThree/method1/intermediates_method1_n_1_c_3.txt',False)
#kmeans('finalThree/method1/intermediates_method1_n_1_c_4.txt',False)
#kmeans('finalThree/method1/intermediates_method1_n_1_c_5.txt',False)

#Method2
#kmeans('finalThree/method2/avg_dist_c3.txt',False)
#kmeans('finalThree/method2/avg_dist_c4.txt',False)
#kmeans('finalThree/method2/avg_dist_c5.txt',False)



#Method3
#kmeans('finalThree/method3.2/intermediates_method_3_avg_dist_and_angle_c_3.txt',False)
#kmeans('finalThree/method3.2/intermediates_method_3_avg_dist_and_angle_c_4.txt',False)
#kmeans('finalThree/method3.2/intermediates_method_3_avg_dist_and_angle_c_5.txt',False)
#kmeans('finalThree/method3.2/convex_hull_all.txt',True)
#kmeans('finalThree/method3.2/convex_hull_c_4.txt',True)
#kmeans('finalThree/method3.2/convex_hull_c_5.txt',True)