#imports
import numpy as np
from scipy.spatial import ConvexHull

#This function takes in a set of points, makes a convex hull out of these points,
# then calculte and returns the volume of this convex hull
def calculate_volume(points):
    hull = ConvexHull(points)
    
    # Get the vertices and simplices of the convex hull
    vertices = hull.points[hull.vertices]
    simplices = hull.simplices
    
    # Calculate the volume using the decomposition into tetrahedra
    volume = 0.0
    for simplex in simplices:
        tetrahedron = vertices[simplex]
        signed_volume = np.dot(tetrahedron[0], np.cross(tetrahedron[1]-tetrahedron[0], tetrahedron[2]-tetrahedron[0]))
        volume += signed_volume / 6.0
    
    return abs(volume)
    

#This function does a convexhull analysis. It takes in the file (readfile) to perform this analysis,
# then writes all the results too the writefile, the size 4 intermediates to writefile2,
# the size 5 intermediates to writefile3. Some intermediates can not be used to perform a convexhull
# analysis, these will be written to the errorfile.
def analyze(readfile,writefile,writefile2,writefile3,errorfile):
    with (open(writefile,'w') as f2, open(writefile2,'w') as f3, open(writefile3,'w') as f4,
     open(errorfile,'w') as f5):
        for i in range(1,101):
            print(i)
            readfile1=readfile+str(i)+'_'+str(i+1)+'.txt'
            with open(readfile1,'r') as f:
                for line in f:
                    points=[]
                    molecules=np.array(eval(line),float)
                    molecules=molecules.tolist()
                    ids=[]
                    for mol in molecules:
                        id=int(mol.pop(0))
                        points.append(mol)
                        ids.append(id)
                    if len(points)>3:
                        try:
                            calculate_volume(points)
                        except:
                            print(points)
                            f5.write(str(len(points))+': '+line)
                        else:
                            volume=calculate_volume(points)
                            density= len(ids)/volume
                            info=[len(ids),ids,volume,density,i] # i indicates the frame 
                            f2.write(str(info)+'\n')
                            if len(ids)==4:
                                f3.write(str(info)+'\n')
                            if len(ids)==5:
                                f4.write(str(info)+'\n')


#analyze('finalThree/method1/analyzed_intermediates_', 'finalThree/method1/convex_hull_all.txt',
#'finalThree/method1/convex_hull_c_4.txt', 'finalThree/method1/convex_hull_c_5.txt',
#'finalThree/method1/convex_hull_errors.txt')

